@@ -14,20 +14,23 @@ Having it into a standalone module allows running the program without requiring
 from __future__ import annotations
 
 from pathlib import Path
-from typing import Any, Dict, List, Optional, Union
+from typing import Any
 
-from .storage import CacheStorage, Snapshot
+from webchanges.handler import Snapshot
+from webchanges.storage import SsdbStorage
 
 try:
     import minidb
-except ImportError:
+except ImportError as e:  # pragma: no cover
 
     class minidb:  # type: ignore[no-redef]
         class Model:
             pass
 
+    minidb_error = str(e)
 
-class CacheMiniDBStorage(CacheStorage):
+
+class SsdbMiniDBStorage(SsdbStorage):
     class CacheEntry(minidb.Model):
         guid = str
         timestamp = int
@@ -35,11 +38,11 @@ class CacheMiniDBStorage(CacheStorage):
         tries = int
         etag = str
 
-    def __init__(self, filename: Union[str, Path]) -> None:
+    def __init__(self, filename: str | Path) -> None:
         super().__init__(filename)
 
         if isinstance(minidb, type):
-            raise ImportError("Python package 'minidb' is missing")
+            raise ImportError(f"Python package 'minidb' cannot be imported.\n{minidb_error}")
 
         self.filename.parent.mkdir(parents=True, exist_ok=True)
 
@@ -50,8 +53,8 @@ class CacheMiniDBStorage(CacheStorage):
         self.db.close()
         self.db = None
 
-    def get_guids(self) -> List[str]:
-        return [guid for guid, in self.CacheEntry.query(self.db, minidb.Function('distinct', self.CacheEntry.c.guid))]
+    def get_guids(self) -> list[str]:
+        return [guid for (guid,) in self.CacheEntry.query(self.db, minidb.Function('distinct', self.CacheEntry.c.guid))]
 
     def load(self, guid: str) -> Snapshot:
         for data, timestamp, tries, etag in self.CacheEntry.query(
@@ -61,12 +64,12 @@ class CacheMiniDBStorage(CacheStorage):
             where=self.CacheEntry.c.guid == guid,
             limit=1,
         ):
-            return Snapshot(data, timestamp, tries, etag)
+            return Snapshot(data, timestamp, tries, etag, '', {})
 
-        return Snapshot('', 0, 0, '')
+        return Snapshot('', 0, 0, '', '', {})
 
-    def get_history_data(self, guid: str, count: Optional[int] = None) -> Dict[str, float]:
-        history: Dict[str, float] = {}
+    def get_history_data(self, guid: str, count: int | None = None) -> dict[str | bytes, float]:
+        history: dict[str | bytes, float] = {}
         if count is not None and count < 1:
             return history
         for data, timestamp in self.CacheEntry.query(
@@ -82,10 +85,10 @@ class CacheMiniDBStorage(CacheStorage):
                     break
         return history
 
-    def get_history_snapshots(self, guid: str, count: Optional[int] = None) -> List[Snapshot]:
+    def get_history_snapshots(self, guid: str, count: int | None = None) -> list[Snapshot]:
         if count is not None and count < 1:
             return []
-        history: List[Snapshot] = []
+        history: list[Snapshot] = []
         for data, timestamp in self.CacheEntry.query(
             self.db,
             self.CacheEntry.c.data // self.CacheEntry.c.timestamp,
@@ -93,57 +96,71 @@ class CacheMiniDBStorage(CacheStorage):
             where=(self.CacheEntry.c.guid == guid)
             & ((self.CacheEntry.c.tries == 0) | (self.CacheEntry.c.tries is None)),
         ):
-            history.append(Snapshot(data, timestamp, 0, ''))
+            history.append(Snapshot(data, timestamp, 0, '', '', {}))
             if count is not None and len(history) >= count:
                 break
         return history
 
-    def save(
-        self,
-        *args: Any,
-        guid: str,
-        data: str,
-        timestamp: float,
-        tries: int,
-        etag: Optional[str],
-        **kwargs: Any,
-    ) -> None:
-        self.db.save(self.CacheEntry(guid=guid, timestamp=timestamp, data=data, tries=tries, etag=etag))
+    def save(self, *args: Any, guid: str, snapshot: Snapshot, **kwargs: Any) -> None:
+        self.db.save(
+            self.CacheEntry(
+                guid=guid,
+                timestamp=snapshot.timestamp,
+                data=snapshot.data,
+                tries=snapshot.tries,
+                etag=snapshot.etag,
+            )
+        )
         self.db.commit()
 
     def delete(self, guid: str) -> None:
         self.CacheEntry.delete_where(self.db, self.CacheEntry.c.guid == guid)
         self.db.commit()
 
-    def delete_latest(self, guid: str, delete_entries: int = 1) -> int:
+    def delete_latest(self, guid: str, delete_entries: int = 1, **kwargs: Any) -> int:
         raise NotImplementedError("Deleting of latest snapshot not supported by 'minidb' database engine")
 
-    def clean(self, guid: str, keep_entries: int = 1) -> int:
-        if keep_entries != 1:
-            raise NotImplementedError("Only keeping latest 1 entry is supported by 'minidb' database engine")
+    def delete_all(self) -> int:
+        raise NotImplementedError("Deleting of all data not supported by 'minidb' database engine")
 
-        keep_id = next(
-            (
-                self.CacheEntry.query(
-                    self.db,
-                    self.CacheEntry.c.id,
-                    where=self.CacheEntry.c.guid == guid,
-                    order_by=self.CacheEntry.c.timestamp.desc,
-                    limit=1,
-                )
-            ),
-            (None,),
-        )[0]
-
-        if keep_id is not None:
-            result: int = self.CacheEntry.delete_where(
-                self.db, (self.CacheEntry.c.guid == guid) & (self.CacheEntry.c.id != keep_id)
+    def clean(self, guid: str, retain_limit: int = 1) -> int:
+        retain_limit = max(1, retain_limit)
+        keep_ids = [
+            row[0]
+            for row in self.CacheEntry.query(
+                self.db,
+                self.CacheEntry.c.id,
+                where=self.CacheEntry.c.guid == guid,
+                order_by=self.CacheEntry.c.timestamp.desc,
+                limit=retain_limit,
             )
+        ]
+        # If nothing's returned from the query, the given guid is not in the db
+        # and no action is needed.
+        if keep_ids:
+            where_clause = self.CacheEntry.c.guid == guid
+            for keep_id in keep_ids:
+                where_clause = where_clause & (self.CacheEntry.c.id != keep_id)
+            result: int = self.CacheEntry.delete_where(self.db, where_clause)
             self.db.commit()
+            self.db.vacuum()
             return result
 
         return 0
 
+    def move(self, guid: str, new_guid: str) -> int:
+        total_moved = 0
+        if guid != new_guid:
+            # Note if there are existing records with 'new_guid', they will
+            # not be overwritten and the job histories will be merged.
+            for entry in self.CacheEntry.load(self.db, self.CacheEntry.c.guid == guid):
+                entry.guid = new_guid
+                entry.save()
+                total_moved += 1
+            self.db.commit()
+
+        return total_moved
+
     def rollback(self, timestamp: float) -> None:
         raise NotImplementedError("Rolling back of legacy 'minidb' databases is not supported")
 
